//Program to analyse frequency data from a neutral marker experiment.  A number of mutants (defined by time, marker population, and selection 
//coefficient) are optimised to best reproduce a set of experimental frequencies.  Two variables are input to the code - a random seed and the 
//number of mutants.  Optimisation proceeds by testing random start points, then running a basic annealing process from the best of these

//The following code has been thoroughly tested for the case with 2 marker populations.

#include <iostream>
#include <vector>

using namespace std;

#include "shared_new.h"

vector<mutant> mutations;
vector<mutant> mutations_search;

vector<m_pop> mpops;
vector<m_pop> new_mpops;

int main(int argc, char *argv[]){
	
	//Set up random number generator
	int seed=atoi(argv[1]);   //First command line input - seed for the random number generator
	gsl_rng_env_setup();
	gsl_rng *rgen = gsl_rng_alloc (gsl_rng_taus);
	gsl_rng_set (rgen, seed);

	//Initial parameters
	int n_mpops=2;  //Number of marker populations
	int sample_stop=641;  //Limit on the number of generations sampled 
	
	vector<int> real_times;  //Stores the times at which the marker frequency was measured
	vector<d_vec> real_counts;  //Stores the measurements of the marker population sizes
	int last_gen=0;

	//Read in marker population data from file specified in argv[4]
	int quiet=0;  //Determines output of 'Real_frequency.out' file, equivalent to the input file but with counts converted to frequencies
	int max_pop_size=0;
	const char* name=argv[4]; //Fourth command line input - the name of the file containing the marker population sizes with time.
	SamplePopulationData (name,n_mpops,last_gen,sample_stop,quiet,max_pop_size,real_times,real_counts);
	
	//Parameters for the optimisation
	int n_mut=atoi(argv[2]); //Second command line input - number of mutants
	gsl_vector *mut_info = gsl_vector_calloc(2*n_mut);
	int reps=atoi(argv[3]); //Third command line input - number of times the optimisation process is repeated within a single calculation
	
	//Create stored vector of values of log N! Used to speed up the code
	vector<double> fact_store;
	find_logfact(fact_store,max_pop_size+1);
	
	vector<mut_data> mutation_store; //Structure containing mutation data from each repitition
		
	int status;
	double size;
	double best_fit=10000000000;
	double last_fit=10000000000;
	double step=0;
	int fail = 0;
	vector<double> start_freq;
	vector<double> start_freq_search;
	int verbose=0;  //Gives additional output if set to 1
	int standing=0;	 //Allows for the possibility of standing variation when set to 1
	int systematic=0;  //Triggers systematic search - variable required but not used here
	double new_mut_size=0.001;  //Size of a new haplotype as a fraction of the total population size
	double fm_size;  //Parameter for standing variation - defines the size of the first mutant haplotype in the population at time zero.
	double fm_ss;
	if (standing==1) {
		fm_size=0.001;
		fm_ss=fm_size;
	} else {
		fm_size=0;
		fm_ss=fm_size;
	}
	int find_mean=0;  //Returns the mean log likelihood over the sampling points rather than the total log likelihood.  This in theory could be used for fitting a limited number of mutants to the first t' generations of the simulation, where t' is variable.  The optimial t' would indicate the point beyond which the trajectory did not fit the limited mutant solution.  This option was never used in the paper, but is left as a historical remnant of the code.
	int p_size;

	vector<int>index;
	int n_reported=10;  //Number of sets of results for which parameters are outputted

	for (int r=1; r<=reps; r++) {
		cout << "Repetition " << r << "\n"; 
		GenerateInitialMutants (n_mut,n_mpops,last_gen,rgen,mutations);  //Random initial mutation parameters
 	
		mutations_search=mutations; //Temporary structure used in optimisation. 
	
		//General setup for optimisation routine
		best_fit=10000000000;  
		last_fit=10000000000;
		step=0;
		fail = 0;
		if (r>1) {
		start_freq.clear();
		start_freq_search.clear();
		}
		for (int i=0;i<n_mpops;i++){
			start_freq.push_back(0);
			start_freq_search.push_back(0);
		}

		new_mut_size=0.001;
		find_mean=0;
		if (standing==1) {
			fm_size=0.001;
			fm_ss=fm_size;
		}
		
		int x_size=0;
		if (standing==1) {
			x_size=(2*n_mut)+n_mpops;
		} else {
			x_size=(2*n_mut)+n_mpops-1;
		}
		
		//Optimisation processs
		for (int it=0;it<1000000;it++){
			size_t iter=0; //GSL iteration number

			//Process result of the last optimisation
			CheckLastLikelihood (n_mut,n_mpops,fail,step,best_fit,last_fit,fm_size,fm_ss,index,start_freq,start_freq_search,mutations,mutations_search,mut_info,standing,systematic);

			if ((fail > n_mut * 100)&&(step<5)) {  //Signals the end of the optimisation process
				mut_data dat;
				dat.data=mutations;
				dat.sf=start_freq;
				dat.logL=best_fit;
				if (standing==1) {
					dat.fm_size=fm_size;
				}
				mutation_store.push_back(dat);  //Optimised values pushed to storage
				break;
			}
		
			//Next optimisation step 
			FirstOptimisationStep (it,fail,n_mut,n_mpops,last_gen,step,mutations,mutations_search,rgen);
			
			//Vector x contains parameters to be optimised
			gsl_vector *x = gsl_vector_calloc(x_size);
			SetXVector (n_mut,n_mpops,fm_ss,mutations_search,x,verbose,standing,systematic);

			//Find the time order of mutations
			for (int i=0;i<n_mut;i++) {
				int less=0;
				for (int j=0;j<n_mut;j++) {
					if (mutations_search[j].time<mutations_search[i].time) {
						less++;
					}
				}
				mutations_search[i].order=less;
			}
						
			//GSL minimisation process follows.  This takes as input a set of fixed parameters, params, and a vector of parameters to optimise, x.
	
			//Define the optimisation parameters
			
			p_size = real_times.size()+(n_mpops*real_counts.size()) + (2*n_mut) + 15;
			int temp=p_size+1;
			p_size=p_size+max_pop_size+1;
			
			double *params;
			params=(double *)calloc(p_size,sizeof(double));
			
			SetParams (params,p_size,n_mut,n_mpops,last_gen,temp,find_mean,max_pop_size,systematic,standing,new_mut_size,real_counts,mutations_search,real_times,fact_store);

			//Define the optimisation function
			gsl_multimin_function my_func;
			my_func.n=x_size;
			my_func.f=&minimise_freqs_new;
			my_func.params= params;
						
			gsl_multimin_fminimizer *s;
			const gsl_multimin_fminimizer_type *T=gsl_multimin_fminimizer_nmsimplex;
			s=gsl_multimin_fminimizer_alloc(T,x_size);

			gsl_vector* ss=gsl_vector_alloc(x_size);
			for (int i=0;i<(2*n_mut)+(n_mpops-1);i++){
				if ((i<(2*n_mut))&&(i%2==1)){
					gsl_vector_set(ss,i,1);
				} else {
					gsl_vector_set(ss,i,0.1);
				}
			}
			if (standing==1) {
				gsl_vector_set(ss,(2*n_mut)+n_mpops-1,0.1);
			}
		
			gsl_multimin_fminimizer_set (s,&my_func,x,ss);
	
			do {
				iter++;
				status=gsl_multimin_fminimizer_iterate(s);
				if (status) {break;}
		
				size=gsl_multimin_fminimizer_size (s);
				status = gsl_multimin_test_size(size, 1e-4);
		
				if (status == GSL_SUCCESS) {
				}
		
			} while (status==GSL_CONTINUE && iter<100);
		
			last_fit = s->fval;	
			if (verbose==1) {
				cout << "Last fit score = " << last_fit << " at (";
			}
		
			//Start frequencies
			for (int c=0;c<n_mpops-1;c++){
				start_freq_search[c]=gsl_vector_get(s->x,(2*n_mut)+c);
			}
		
			//Mutant parameters
			for (int m=0;m<n_mut;m++){
				gsl_vector_set(mut_info,(2*m),gsl_vector_get(s->x, (2*m)));
				gsl_vector_set(mut_info,(2*m)+1,gsl_vector_get(s->x, (2*m)+1));
			};
			
			//Initial mutant size
			if (standing==1) {
				fm_ss=gsl_vector_get(s->x,(2*n_mut)+n_mpops-1);
			}
			
			gsl_vector_free(x);
			gsl_vector_free(ss);
			gsl_multimin_fminimizer_free(s);
			
		}
		
			
	}
	//Output result of each optimisation
	cout << "Repetitions = " << reps << "\n";
	for (int i=0;i<reps;i++) {
		cout << "Run" << i << " Log L = " << mutation_store[i].logL << "\n";
	}
	//Sort mutations by log likelihood
	vector<mut_data> mutation_store_sorted;
	for (int i=0;i<10;i++) {
		double min_l=10000000;
		int index=-1;
		for (int j=0;j<reps;j++) {
			if (mutation_store[j].logL<min_l) {
				index=j;
				min_l=mutation_store[j].logL;
			}
		}
		mut_data mut=mutation_store[index];
		mutation_store_sorted.push_back(mut);
		mutation_store[index].logL=10000000;
	}
			
	//Write output of the best n_reported optimisations
	WriteTransferDataFile (n_reported,n_mut,n_mpops,mutation_store_sorted,standing);
	
	return EXIT_SUCCESS;
}
	
	
