//Program to analyse frequency data from a neutral marker experiment.  A number of mutants (defined by time, marker population, and selection 
//coefficient) are optimised to best reproduce a set of experimental frequencies.  Two variables are input to the code - a random seed and the 
//number of mutants.  Optimisation proceeds by systematic search, beginning from a pre-defined point

#include <iostream>
#include <vector>

using namespace std;

#include "shared_new.h"

vector<mutant> mutations;
vector<mutant> mutations_search;

vector<m_pop> mpops;
vector<m_pop> new_mpops;

int main(int argc, char *argv[]){
	
	//Set up random number generator
	int seed=atoi(argv[1]);
	gsl_rng_env_setup();
	gsl_rng *rgen = gsl_rng_alloc (gsl_rng_taus);
	gsl_rng_set (rgen, seed);

	//Initial parameters
	int n_mut;
	vector<mutant> mutations;
	int n_mpops;
	vector<double> start_freq;
	double best_fit;
	GetTransferData (n_mut,mutations,n_mpops,start_freq,best_fit);  //Reads in mutation data from a previous optimisation
	best_fit=1000000;

	for (int m=0;m<n_mut;m++){
		cout << "Initial mutation " << m << " " << mutations[m].selection << " " << mutations[m].time << " " << mutations[m].mpop << "\n";
	}
	
	vector<int> real_times;
	vector<d_vec> real_counts;
	int last_gen=0;
	int sample_stop=641;
	
	//Read in marker population data from Colour_freq.out - this actually contains raw numbers of marker population counts
	int quiet=1;
	int max_pop_size=0;
	const char* name=argv[2];
	SamplePopulationData (name,n_mpops,last_gen,sample_stop,quiet,max_pop_size,real_times,real_counts);

	gsl_vector *mut_info = gsl_vector_calloc(n_mut);
	mutations_search=mutations; 
	
	//Create stored vector of values of log N!
	vector<double> fact_store;
	find_logfact(fact_store,max_pop_size+1);

	int status;
	double size;
	double last_fit=10000000000;
	vector<double> start_freq_search;
	for (int i=0;i<n_mpops;i++){
		start_freq_search.push_back(0);
	}

	double new_mut_size=0.001;
	int find_mean=0;
	vector<int>change;
	vector<int>index;
	int alt=1;	//Stops overwriting of previous output file - writes to Transfer_data2.out
	int verbose=0;   //Controls some additional output commands
	int standing=0;  //Optional switch for including standing variation
	int systematic=1; //Here invokes systematic search options throughout the code
	double step=0;
	int fail = 0;
	double fm_size;
	if (standing==1) {
		fm_size=0.001;
	} else {
		fm_size=0;
	}
	double fm_ss=fm_size;
	int x_size;
	int p_size;
	
	for (int run=1;run<=4;run++) {
		change.clear();  //Change stores possible moves in systematic search
		index.clear(); //Index details which moves have currently been made
		
		//Derive systematic search parameters for this run
		SystematicSearchParameters(run,change);
		
		InitialSystematicOutput (n_mut,change,index);
	
		int finish=0;
		int ok=0;
		int it=0;
		if (standing==1) {
			x_size=n_mut+n_mpops;
		} else {
			x_size=n_mut+n_mpops-1;
		}
		//Begin optimisation process
		do {
			it++;
			size_t iter=0; //GSL iteration number
			
			if (it==1) {
				cout << "Current underlying data\n";
				for (int m=0;m<n_mut;m++){
					cout << "Mutation " << m << " " << mutations[m].selection << " " << mutations[m].time << " " << mutations[m].mpop << "\n";
				}		
			}
		
			//Process result of the last optimisation
			CheckLastLikelihood (n_mut,n_mpops,fail,step,best_fit,last_fit,fm_size,fm_ss,index,start_freq,start_freq_search,mutations,mutations_search,mut_info,standing,systematic);

			//Next optimisation step 
			SystematicSearchStep (finish,ok,it,n_mut,last_gen,index,change,mutations,mutations_search);
			
			//Vector x contains parameters for the optimisation
			gsl_vector *x = gsl_vector_calloc(x_size); //Variables for GSL optimisation
	 		SetXVector (n_mut,n_mpops,fm_ss,mutations_search,x,verbose,standing,systematic);
				
			//Find order of mutations
			for (int i=0;i<n_mut;i++) {
				int less=0;
				for (int j=0;j<n_mut;j++) {
					if (mutations_search[j].time<mutations_search[i].time) {
						less++;
					}
				}
				mutations_search[i].order=less;
			}			
			
			//GSL minimisation process follows
	
			//Define the optimisation parameters
			p_size = real_times.size()+(n_mpops*real_counts.size()) + (n_mut*3) + 15;
			int temp=p_size+1;
			p_size=p_size+max_pop_size+1;

			double *params;
			params=(double *)calloc(p_size,sizeof(double));
			
			SetParams (params,p_size,n_mut,n_mpops,last_gen,temp,find_mean,max_pop_size,systematic,standing,new_mut_size,real_counts,mutations_search,real_times,fact_store);

			//Define the optimisation function
			gsl_multimin_function my_func;
			my_func.n=x_size;
			my_func.f=&minimise_freqs_new;
			my_func.params= params;
	
			gsl_multimin_fminimizer *s;
			const gsl_multimin_fminimizer_type *T=gsl_multimin_fminimizer_nmsimplex;
			s=gsl_multimin_fminimizer_alloc(T,x_size);

			gsl_vector* ss=gsl_vector_alloc(x_size);
			for (int i=0;i<n_mut+(n_mpops-1);i++){
				gsl_vector_set(ss,i,0.1);
			}
			if (standing==1) {
				gsl_vector_set(ss,n_mut+n_mpops-1,0.1);
			}
			
			gsl_multimin_fminimizer_set (s,&my_func,x,ss);
	
			do {
				iter++;
				status=gsl_multimin_fminimizer_iterate(s);
				if (status) {break;}
		
				size=gsl_multimin_fminimizer_size (s);
				status = gsl_multimin_test_size(size, 1e-4);
		
				if (status == GSL_SUCCESS) {
				}
		
			} while (status==GSL_CONTINUE && iter<100);
		
			last_fit = s->fval;	
			if (verbose==1) {
				cout << "Last fit score = " << last_fit << " at (";
			}
			
			//Start frequencies
			for (int c=0;c<n_mpops-1;c++){
				start_freq_search[c]=gsl_vector_get(s->x,n_mut+c);
			}
		
			//Mutant parameters
			for (int m=0;m<n_mut;m++){
				gsl_vector_set(mut_info,m,gsl_vector_get(s->x, m));
			};	
			
			//Initial mutant size
			if (standing==1) {
				fm_ss=gsl_vector_get(s->x,n_mut+n_mpops-1);
			}
			
			gsl_vector_free(x);
			gsl_vector_free(ss);
			gsl_multimin_fminimizer_free (s);
	
		} while (finish==0);
		mutations_search=mutations;
	}
	//Write output of the converged systematic search
	DoFinalOutput (alt,new_mut_size,best_fit,fm_size,n_mut,last_gen,n_mpops,find_mean,mutations,real_counts,real_times,start_freq,fact_store,standing);

	
	return EXIT_SUCCESS;
}
	
	
