#include "shared_new.h"
#include <iostream>
#include <string>

//Reads in counts of marker population size from a given file 
void SamplePopulationData (const char* name, int n_mpops, int& last_gen, int sample_stop, int quiet, int& max_pop_size, vector<int>& real_times, vector<d_vec>& real_counts) {
	//Find real selection coefficients
	ofstream rf;
	if (quiet==0) {
		rf.open("Real_frequency.out");
	} 
	
	//Read in colour frequency data
	ifstream colour_file;
	colour_file.open(name);
	int t=0;
	vector<double> counts;
	int x_int;
	int x_sum;
	
	while (t<sample_stop){
		last_gen=t;
		counts.clear();
		x_sum=0;
		colour_file >> t;
		for (int c=1;c<=n_mpops;c++){
			colour_file >> x_int;
			counts.push_back(x_int);
			x_sum=x_sum+x_int;
		}
		if (x_sum>max_pop_size) {
			max_pop_size=x_sum;
		}
		
		if (quiet==0) {
			rf << t << " ";
			for (int c=0;c<n_mpops;c++){
				rf << counts[c]/x_sum << " ";
			}
			rf << "\n";
		}
		d_vec d;
		d.count=counts;
		real_counts.push_back(d);
		real_times.push_back(t);
		if (max(counts[0],counts[1])<1) {break;}
		
	}	
	cout << "Last generation is " << last_gen << "\n";
	if (quiet==0) {
		rf.close();
	}	
}	

//Creates an lookup table for log N!  Used to speed up the code
void find_logfact(vector<double>& fact_store, int N){
	double logN=0;
	fact_store.push_back(0);
	for (int i=1;i<=N;i++) {
		logN=logN+log(i);
		fact_store.push_back(logN);
	}
}

//Define the step sizes for a systematic search
void SystematicSearchParameters(int run, vector<int>& change) {
	change.push_back(0);
	if (run==1) {
		change.push_back(29);
		change.push_back(-29);
		change.push_back(17);
		change.push_back(-17);
		change.push_back(7);
		change.push_back(-7);
		change.push_back(2);
		change.push_back(-2);
	}
	if (run==2) {
		change.push_back(23);
		change.push_back(-23);
		change.push_back(13);
		change.push_back(-13);
		change.push_back(5);
		change.push_back(-5);
		change.push_back(1);
		change.push_back(-1);
	}
	if (run==3) {
		change.push_back(19);
		change.push_back(-19);
		change.push_back(11);
		change.push_back(-11);
		change.push_back(3);
		change.push_back(-3);
		change.push_back(1);
		change.push_back(-1);
	}
	if (run==4) {
		change.push_back(4);
		change.push_back(-4);
		change.push_back(3);
		change.push_back(-3);
		change.push_back(2);
		change.push_back(-2);
		change.push_back(1);
		change.push_back(-1);
	}
}	

//Generates an initial set of mutants, with random times, selection coefficients, and marker populations
void GenerateInitialMutants (int n_mut, int n_mpops, int last_gen, gsl_rng *rgen, vector<mutant>& mutations) {
	for (int m=1;m<=n_mut;m++){
		mutant mut;
		mut.time = gsl_rng_uniform(rgen)*(last_gen-1);  //Time at which mutation occurred
		mut.mpop = floor(gsl_rng_uniform(rgen)*(n_mpops));  //Marker population in which the mutation occurred
		mut.selection = (gsl_rng_uniform(rgen)*0.5);  //Selection coefficient of the mutation (always positive)
		mutations.push_back(mut);
	}
}	

//Compares the previous log likelihood to the best log likelihood obtained so far.  If the new log likelihood is the best so far, the parameters from the last run are stored in memory
void CheckLastLikelihood (int n_mut, int n_mpops, int& fail, double step,  double& best_fit, double& last_fit, double& fm_size, double fm_ss, vector<int>& index, vector<double>& start_freq, vector<double>& start_freq_search, vector<mutant>& mutations, vector<mutant>& mutations_search, gsl_vector *mut_info, int standing, int systematic) {
	if (last_fit<best_fit){  //Check for improvement in log likelihood
		best_fit=last_fit;
		cout << "New likelihood = " << best_fit << "\n";
		mutations=mutations_search;
		
		if (systematic==0) {
			for (int m=0;m<n_mut;m++){
				mutations[m].selection=gsl_vector_get(mut_info,(2*m));
				mutations[m].time=gsl_vector_get(mut_info,(2*m)+1);
			}
			fail=0;
		}
		if (systematic==1) {
			for (int m=0;m<n_mut;m++){
				mutations[m].selection=gsl_vector_get(mut_info,m);
			}
			//Reset the index
			for (unsigned int i=0;i<index.size();i++){
				index[i]=0;
			}
			index[0]=-1;
		}
		
		cout << "Now set at:\n";
		for (int m=0;m<n_mut;m++){
			cout << "Mutation " << m << " " << mutations[m].selection << " " << mutations[m].time << " " << mutations[m].mpop << "\n";
		}
		
		//Start frequencies
		double last=1;
		for (int c=0;c<n_mpops-1;c++){
			start_freq[c]=start_freq_search[c];
			last=last-start_freq[c];
		}
		start_freq[n_mpops-1]=last;
		
		if (standing==1) {
			fm_size=fm_ss;
		}
		
	} else {
		if (systematic==0) {
			//Revert to old configuration
			fail++;
			if ((fail > n_mut * 100)&&(step<5)) {
				mutations_search=mutations;
			}
		}
	}
}

//Set initial values for x
void SetXVector (int n_mut, int n_mpops, double fm_ss, vector<mutant>& mutations_search, gsl_vector *x, int verbose, int standing, int systematic) {
	if (verbose==1) {
		cout << "New Trial:\n";
	}
	
	if (systematic==0) {
		for (int m=0;m<n_mut;m++){
			gsl_vector_set(x,(2*m),0.1);  //Begin search from a constant selection coefficient 
			gsl_vector_set(x,(2*m)+1,mutations_search[m].time);
			if (verbose==1) {
				cout << "Mutation " << m << " " << mutations_search[m].selection << " " << mutations_search[m].time << " " << mutations_search[m].mpop << "\n";
			}
		}
		//Initial marker frequencies
		for (int c=0;c<n_mpops-1;c++){
			gsl_vector_set(x,(2*n_mut)+c,1/(n_mpops+0.));
		}
	
		//Size of first mutation initial population
		if (standing==1) {
			gsl_vector_set(x,(2*n_mut)+n_mpops-1,fm_ss);
		}
	}
	
	if (systematic==1) {  //Systematic search gets mutation times from the main code...
		for (int m=0;m<n_mut;m++){
			gsl_vector_set(x,m,0.1);  //Begin search from a constant selection coefficient 
			if (verbose==1) {
				cout << "Mutation " << m << " " << mutations_search[m].selection << " " << mutations_search[m].time << " " << mutations_search[m].mpop << "\n";
			}
		}
		//Initial marker frequencies
		for (int c=0;c<n_mpops-1;c++){
			gsl_vector_set(x,n_mut+c,1/(n_mpops+0.));
		}
		
		//Size of first mutation initial population
		if (standing==1) {
			gsl_vector_set(x,n_mut+n_mpops-1,fm_ss);
		}
	}
}

//Set parameters for a step of the GSL optimisation routine
void SetParams (double *params, int p_size, int n_mut,int n_mpops, int last_gen, int temp, int find_mean, int max_pop_size, int systematic, int standing, double new_mut_size, vector<d_vec> real_counts, vector<mutant> mutations_search, vector<int> real_times, vector<double> fact_store) {
	params[0]=n_mut;	//General parameters
	params[1]=n_mpops;
	params[2]=last_gen;
	params[3]=temp;
	params[4]=real_counts.size();
	params[5]=new_mut_size;
	params[6]=find_mean;
	params[7]=max_pop_size;
	params[8]=systematic;
	params[9]=standing;
	int m_i=0;
	
	if (systematic==0) {
	
		for (int i=10;i<10+(2*n_mut);i=i+2){
			params[i]=mutations_search[m_i].mpop;  //Marker populations of mutants
			params[i+1]=mutations_search[m_i].order; //Time ordering of mutations
			m_i++;
		}
	
	
		int s_i=0;
		for (unsigned int i=10+(2*n_mut);i<10+(2*n_mut)+real_times.size();i++){  //Times at which frequencies were observed
			params[i]=real_times[s_i];
			s_i++;
		}
	
		int t_index=0;
		unsigned int i=10+(2*n_mut)+real_times.size();
		do {
			for (int c=0;c<n_mpops;c++) {
				params[i+c]=real_counts[t_index].count[c];  //Frequencies observed at each time point
			}
			i=i+n_mpops;
			t_index++;
		} while (i<10+(n_mpops*real_counts.size())+(2*n_mut)+real_times.size());
		for (int i=temp;i<=p_size;i++){
			params[i]=fact_store[i-temp+1];
		}
		
	}
	
	if (systematic==1) {
		params[0]=n_mut;	//General parameters
		params[1]=n_mpops;
		params[2]=last_gen;
		params[3]=temp;
		params[4]=real_counts.size();
		params[5]=new_mut_size;
		params[6]=find_mean;
		params[7]=max_pop_size;
		params[8]=systematic;
		params[9]=standing;
		int m_i=0;
		for (int i=10;i<10+(n_mut*3);i=i+3){
			params[i]=mutations_search[m_i].time;  //Emergence times of mutants
			params[i+1]=mutations_search[m_i].mpop;  //Marker populations of mutants
			params[i+2]=mutations_search[m_i].order;  //Time ordering of mutations
			m_i++;
		}
		
		int s_i=0;
		for (unsigned int i=10+(n_mut*3);i<10+(n_mut*3)+real_times.size();i++){  //Times at which frequencies were observed
			params[i]=real_times[s_i];
			s_i++;
		}
		
		int t_index=0;
		unsigned int i=10+(n_mut*3)+real_times.size();
		do {
			for (int c=0;c<n_mpops;c++) {
				params[i+c]=real_counts[t_index].count[c];  //Frequencies observed at each time point
			}
			i=i+n_mpops;
			t_index++;
		} while (i<10+(n_mpops*real_counts.size())+(n_mut*3)+real_times.size());
		for (int i=temp;i<=p_size;i++){
			params[i]=fact_store[i-temp+1];
		}
	}		
}
	

//Sets parameters for the GSL optimisation routine in the 0-mutant case.
void SetParamsNoMut (double *params, int n_mpops, int last_gen, int seed, vector<d_vec> real_counts, vector<int> real_times) {
	params[0]=n_mpops;
	params[1]=last_gen;
	params[2]=seed;
	params[3]=real_counts.size();
	
	int s_i=0;
	for (unsigned int i=4;i<4+real_times.size();i++){  //Times at which frequencies were observed
		params[i]=real_times[s_i];
		s_i++;
	}
	
	int t_index=0;
	unsigned int i=4+real_times.size();
	do {
		for (int c=0;c<n_mpops;c++) {
			params[i+c]=real_counts[t_index].count[c];  //Frequencies observed at each time point
		}
		i=i+n_mpops;
		t_index++;
	} while (i<4+(n_mpops*real_counts.size())+real_times.size());
}	
	


//Run a step of the non-sytematic optimisation process - random search followed by annealing process
void FirstOptimisationStep (int it, int fail, int n_mut, int n_mpops, int last_gen, double& step, vector<mutant>& mutations, vector<mutant>& mutations_search, gsl_rng *rgen) {
	double f_prob;
	double flip;
	
	if (it<n_mut*100){  //Early on, generate start points at random
		for (int m=0;m<n_mut;m++){
			mutations_search[m].time = gsl_rng_uniform(rgen)*(last_gen-1);  //Time at which mutation occurred
		}
		fail=0;
	} else { //Next, stick within a decreasing window of the best identified start point
		for (int m=0;m<n_mut;m++){
			do {
				step=max(3.0,(last_gen/4)*pow(0.99,((it-(n_mut*50))/(n_mut+0.))));
				mutations_search[m].time = mutations[m].time + gsl_rng_uniform(rgen)*((2*step)+1)-step;
			} while ((mutations_search[m].time<0)||(mutations_search[m].time>last_gen));
		}
	}
	if (step>15) {  //Later on in optimisation stop flipping the marker populations - assume they are optimised by this point
		f_prob = 1-pow(0.9,n_mut);
		for (int m=0;m<n_mut;m++){
			flip = gsl_rng_uniform(rgen);
			if (flip<f_prob) {
				mutations_search[m].mpop=floor(gsl_rng_uniform(rgen)*n_mpops);
			} else {
				mutations_search[m].mpop=mutations[m].mpop;
			}
		}
	}
}

//Perform a step of the systematic search process
void SystematicSearchStep (int& finish, int ok, int it, int n_mut, int last_gen, vector<int>& index, vector<int>& change, vector<mutant>& mutations, vector<mutant>& mutations_search) {
	if (it>1) {
		int chk;
		int m;
		do {
			//Make the step
			for (chk=0;chk<n_mut;chk++){
				if (index[chk]>=8){
					if (chk==n_mut-1) {//Done search
						finish=1;
						break;
					} else {
						index[chk]=0;
						index[chk+1]++;
						if (index[chk+1]<=8) {
							break;
						}
					}
				} else {
					index[chk]++;
					break;
				}
			}
			//Check not all zeros
			ok=0;
			for (chk=0;chk<n_mut;chk++){
				if (index[chk]!=0) {
					ok=1;
				}
			}
			//Define the mutation times
			for (m=0;m<n_mut;m++){
				mutations_search[m].time=mutations[m].time+change[index[m]];
				if ((mutations_search[m].time<0)||(mutations_search[m].time>last_gen)) {
					ok=0;
					break;
				}
				mutations_search[m].selection=mutations[m].selection;
			}
		} while ((ok==0)&&(finish==0));
	} else {
		mutations_search=mutations; 
	}
}

//Return parameters to the optimisation routine
void GetParams(double *p, int& bk, int& n_mut, int& n_mpops, int& last_gen, int& ps, int& rss, double& new_mut_size, int& find_mean, int& max_pop_size, int& systematic, int& standing, double& fm_size, double& logL, vector<mutant>& mutations, vector<int>& real_times, vector<d_vec>& real_counts, vector<double>& fact_store, vector<double>& start_freqs, const gsl_vector *x) {
	n_mut=floor(p[0]+0.001);  //Number of mutations  
	n_mpops=floor(p[1]+0.001); //Number of mpops
	last_gen=floor(p[2]+0.001); //Last generation in sample 
	ps=floor(p[3]+0.001);  //Marks a convenient location within the parameter vector
	rss=floor(p[4]+0.001);  //Size of real_sel vector
	new_mut_size=p[5]; //Size of new mutant population
	find_mean=p[6];
	max_pop_size=floor(p[7]+0.001);
	systematic=floor(p[8]+0.001);
	standing=floor(p[9]+0.001);
	fm_size=0;
	
	if (systematic==0) {
		
		if (standing==1) {
			fm_size=gsl_vector_get(x,(2*n_mut)+n_mpops-1);
			if ((fm_size<0.001)||(fm_size>0.999)) {
				logL=100000000000000000;
				bk=1;
			}
		}
		
		if (bk==0) {
			for (int m=0;m<n_mut;m++){
				mutant mut;
				mut.mpop = p[(2*m)+10];
				mut.order= p[(2*m)+11];
				mut.selection = gsl_vector_get(x,2*m);
				mut.time = gsl_vector_get(x,(2*m)+1);
				if ((mut.selection<0)||(mut.time>last_gen)||(mut.time<0)) {	//Must have +ve selection, mutation within simulation time
					logL=100000000000000000;
					bk=1;
				}  
				mutations.push_back(mut);
			}
		}
		
		//Prevent switching of the ordering of mutants in the initial optimisation
		if (bk==0) {
			for (unsigned int i=0;i<mutations.size();i++) {
				for (unsigned int j=0;j<mutations.size();j++) {
					if (i!=j) {
						if ((mutations[i].time<mutations[j].time)&&(mutations[i].order>mutations[j].order)) {
							logL=100000000000000000;
							bk=1;
						}
					}
				}
			}
		}
		
		if (bk==0) {
			int i=10+(2*n_mut);
			for (int s=0;s<rss;s++){
				real_times.push_back(p[i]);
				i++;
			}
		
			i=10+(2*n_mut)+rss;
			for (int s=0;s<rss;s++){
				d_vec d;
				for (int c=0;c<n_mpops;c++){
					d.count.push_back(p[i]);
					i++;
				}
				real_counts.push_back(d);
			}
		}
	}
	
	if (systematic==1) {
		
		if (standing==1) {
			fm_size=gsl_vector_get(x,n_mut+n_mpops-1);
			if ((fm_size<0.001)||(fm_size>0.999)) {
				logL=100000000000000000;
				bk=1;
			}
		}
		if (bk==0) {
			for (int m=0;m<n_mut;m++){
				mutant mut;
				mut.time = p[(3*m)+10];
				mut.mpop = p[(3*m)+11];
				mut.order= p[(3*m)+12];
				mut.selection = gsl_vector_get(x,m);
				if ((mut.selection<0)||(mut.time>last_gen)||(mut.time<0)) {	//Must have +ve selection, mutation within simulation time
					logL=100000000000000000;
					bk=1;
				}  
				mutations.push_back(mut);
			}
		}
		
		if (bk==0) {

			int i=10+(n_mut*3);
			for (int s=0;s<rss;s++){
				real_times.push_back(p[i]);
				i++;
			}
		
			i=10+(n_mut*3)+rss;
			for (int s=0;s<rss;s++){
				d_vec d;
				for (int c=0;c<n_mpops;c++){
					d.count.push_back(p[i]);
					i++;
				}
				real_counts.push_back(d);
			}
		}
	}
	
	if (bk==0) {
		fact_store.push_back(0);
		for (int i=ps;i<=ps+max_pop_size+1;i++){
			fact_store.push_back(p[i]);
		}
	
		//Set initial marker population frequencies
		for (int c=0;c<n_mpops;c++){
			start_freqs.push_back(0);
		}
		start_freqs[n_mpops-1]=1;
	
		if (systematic==0) {
			for (int c=0;c<n_mpops-1;c++){
				start_freqs[c]=gsl_vector_get(x,(2*n_mut)+c);
				start_freqs[n_mpops-1]=start_freqs[n_mpops-1]-start_freqs[c];
			}
		}
		if (systematic==1) {
			for (int c=0;c<n_mpops-1;c++){
				start_freqs[c]=gsl_vector_get(x,n_mut+c);
				start_freqs[n_mpops-1]=start_freqs[n_mpops-1]-start_freqs[c];
			}
		}
	
		if ((start_freqs[n_mpops-1]<=0)||(start_freqs[n_mpops-1]>=1)){ //Start frequencies must be between 0 and 1
			logL=10000000000000000;
			bk=1;
		}
	}
}	

//Within this code, q0 is a vector containing the start frequencies of the different marker populations, followed by the new mutant sizes for each marker population.  The latter (new_mut_size where there is no standing variation) are equivalent to the value q0 from the main text.
void Setq0(int& bk, int n_mpops, int n_mut, int fm_size, int standing, double new_mut_size, double& logL, vector<double>& q0, vector<double> start_freqs, vector<mutant> mutations) {
	for (int c=0;c<n_mpops;c++){
		q0.push_back(start_freqs[c]);
	}
	for (int c=n_mpops;c<n_mpops+n_mut;c++){
		q0.push_back(new_mut_size);
	}
	
	if (standing==1) {
		double min_t=10000;
		int ind=-1;
		for (int i=0;i<n_mut;i++){
			if (mutations[i].time<min_t) {
				min_t=mutations[i].time;
				ind=i;
			}
		}
		if (min_t<1) {  //Allow for a larger starting population if the first mutant appears before time 1
			q0[n_mpops+ind]=fm_size;
			if (fm_size>q0[mutations[ind].mpop]){
				logL=100000000000000000;
				bk=1;
			}   
			q0[mutations[ind].mpop]=q0[mutations[ind].mpop]-fm_size;
		}
	}		
}	

//Routine to calculate the log likelihood of a set of mutation parameters
double minimise_freqs_new (const gsl_vector *x, void *params){
	vector<d_vec> real_counts;
	vector<int> real_times;
	vector<mutant> mutations;
	vector<double> start_freqs;
	vector<double> fact_store;
	vector<double> q0;
	int bk=0;
	int n_mut;    
	int n_mpops; 
	int last_gen;  
	int ps;  
	int rss;  
	double new_mut_size; 
	int find_mean;
	int max_pop_size;
	int systematic;
	int standing;
	double fm_size;
	double *p = (double *)params;
	double logL;

	GetParams(p,bk,n_mut,n_mpops,last_gen,ps,rss,new_mut_size,find_mean,max_pop_size,systematic,standing,fm_size,logL,mutations,real_times,real_counts,fact_store,start_freqs,x);
	
	if (bk==0) {
		//Set the initial frequencies and mutation times
		Setq0(bk,n_mpops,n_mut,fm_size,standing,new_mut_size,logL,q0,start_freqs,mutations);
	}
	if (bk==0) {
		int print_freq=0;
		logL = FindLikelihood_new (print_freq,n_mut,n_mpops,last_gen,find_mean,real_counts,real_times,q0,mutations,fact_store);
	}
	return logL;
}

//Version of the code GetParams for the case in which there are no beneficial mutations
void GetParamsNoMut (double *p, double &logL, int &bk, int& n_mpops, int& last_gen, int& rss, vector<d_vec>& real_counts, vector<int>& real_times, vector<double>& start_freqs, const gsl_vector *x) {
	n_mpops=floor(p[0]+0.001); //Number of mpops
	last_gen=floor(p[1]+0.001); //Last generation in sample 
	rss=floor(p[3]+0.001);  //Size of real_sel vector
	
	int i=4;
	for (int s=0;s<rss;s++){
		real_times.push_back(p[i]);
		i++;
	}
	
	i=4+rss;
	for (int s=0;s<rss;s++){
		d_vec d;
		for (int c=0;c<n_mpops;c++){
			d.count.push_back(p[i]);
			i++;
		}
		real_counts.push_back(d);
	}
	
	//Initial marker population frequencies
	for (int c=0;c<n_mpops;c++){
		start_freqs.push_back(0);
	}
	start_freqs[n_mpops-1]=1;
	for (int c=0;c<n_mpops-1;c++){
		start_freqs[c]=gsl_vector_get(x,c);
		start_freqs[n_mpops-1]=start_freqs[n_mpops-1]-start_freqs[c];
	}
	if ((start_freqs[n_mpops-1]<=0)||(start_freqs[n_mpops-1]>=1)){ //Constraint on initial frequencies x
		logL=10000000000000000;
		bk=1;
	}
}	

//Version of Setq0 for the case in which there are no beneficial mutations
void Setq0NoMut (int n_mpops, vector<double>& q0, vector<double> start_freqs) {
	for (int c=0;c<n_mpops;c++){
		q0.push_back(start_freqs[c]);
	}
}	

//The following routine carries out a minimisation for the 0 mutant case.
double minimise_freqs_nomut (const gsl_vector *x, void *params){
	//cout << "Minimisation routine\n";
	vector<d_vec> real_counts;
	vector<int> real_times;
	vector<mutant> mutations;
	vector<double> start_freqs;
	vector<double> fact_store;
	
	double *p = (double *)params;
	double logL;
	int n_mpops; //Number of mpops
	int last_gen; //Last generation in sample 
	int rss;  //Size of real_sel vector
	
	int bk=0;

	GetParamsNoMut (p,logL,bk,n_mpops,last_gen,rss,real_counts,real_times,start_freqs,x);
	
	if (bk==0) {
	
		//Set the initial frequencies q0
		vector<double> q0;
		Setq0NoMut (n_mpops,q0,start_freqs); 

		find_logfact(fact_store,1000);
		int print_freq=1;
		logL = FindLikelihood_nomut (print_freq,n_mpops,last_gen,real_counts,real_times,q0,fact_store);
	
	}
	
	return logL;
}

//Find z_a for each mutation.  These values are used for normalisation of the nominal size of new haplotypes - see text for more details
void FindZa (int n_mut, vector<mutant>& mutations) {
	int i;
	int j;
	double tot;
	for (int o=0;o<n_mut;o++) {
		for (i=0;i<n_mut;i++) {
			if (mutations[i].order==o) {
				tot=1;
				for (j=0;j<n_mut;j++) {
					if (mutations[j].order<mutations[i].order) {
						tot=tot+((0.001/0.999)*mutations[j].z_init*exp((mutations[i].time-mutations[j].time)*mutations[j].selection));
					}
				}
				mutations[i].z_init=tot;
			}
		}
	}
}

//Find haplotype population sizes at the time t
void FindHaploPops (double& total_q, int t, int n_mpops, int n_mut, vector<double> q0, vector<mutant> mutations, vector<double>& h_pop) {
	total_q=0;
	int h;
	double q;
	for (h=0;h<n_mpops;h++) {  //Original haplotypes
		q=q0[h];
		h_pop.push_back(q);
		total_q=total_q+q;
	}
	for (h=n_mpops;h<n_mpops+n_mut;h++){
		if (t>=mutations[h-n_mpops].time) {
			q=(q0[h]/(1-q0[h]))*mutations[h-n_mpops].z_init*exp(mutations[h-n_mpops].selection*(t-mutations[h-n_mpops].time));
		} else {
			q=0;
		}
		h_pop.push_back(q);
		total_q=total_q+q;
	}
}	

//Same as above for case of no beneficial mutant
void FindHaploPopsNoMut (double& total_q, int t, int n_mpops, vector<double> q0, vector<double>& h_pop) {
	total_q=0;
	double q;
	for (int h=0;h<n_mpops;h++) {  //Original haplotypes
		q=q0[h];
		h_pop.push_back(q);
		total_q=total_q+q;
	}
}	

//Convert haplotype sizes to haplotype frequencies
void FindHaploFreqs (int n_mpops, int n_mut, double total_q, vector<double>& h_pop, vector<double>& h_freq) {
	int h;
	for (h=0;h<n_mpops+n_mut;h++){
		h_freq.push_back(h_pop[h]/total_q);
	}
}	

//Convert haplotype frequencies to marker population frequencies
void FindMarkerFreqs (int t, int print_freq, int n_mpops, int n_mut, ofstream& mf, vector<double> h_pop, vector<double> h_freq, vector<double>& m_freq, vector<mutant> mutations) {
	int h;
	for (h=0;h<n_mpops;h++){
		m_freq.push_back(h_freq[h]);
	}
	for (h=n_mpops;h<n_mpops+n_mut;h++){
		m_freq[mutations[h-n_mpops].mpop]=m_freq[mutations[h-n_mpops].mpop]+h_freq[h];
	}
	
	if (print_freq==1) {
		mf << t << " ";
		for (h=0;h<n_mpops;h++){
			mf << m_freq[h] << " ";
		}
		mf << "\n";
	}
}

//As above for no beneficial mutation case
void FindMarkerFreqsNoMut (int t, int print_freq, int n_mpops, ofstream& mf, vector<double> h_freq, vector<double>& m_freq) {
	for (int h=0;h<n_mpops;h++){
		m_freq.push_back(h_freq[h]);
	}
	
	if (print_freq==1) {
		mf << t << " ";
		for (int h=0;h<n_mpops;h++){
			mf << m_freq[h] << " ";
		}
		mf << "\n";
	}
}

//Calculate a likelihood score
void LikelihoodCalc (int s, int& n_points, double& logL, vector<d_vec> real_counts, vector<double> fact_store, vector<double> m_freq) {
	int N=0;  //Find the number of observations at time t
	unsigned int c;
	double Lt;
	double lp;
	for (c=0;c<real_counts[s].count.size();c++){
		N=N+real_counts[s].count[c];
	}
	
	//Find log multinomial probability...  N!/(x1! x2! x3!)  p_i^(x_i)
	Lt=fact_store[N];
	for (c=0;c<real_counts[s].count.size();c++){
		Lt=Lt-fact_store[real_counts[s].count[c]];
	}
	
	for (c=0;c<real_counts[s].count.size();c++){
		lp=log(m_freq[c]);
		Lt=Lt+(real_counts[s].count[c]*lp);
	}
	logL=logL+Lt;
	n_points=s+1;
}	
	
//Find the log likelihood of a set of mutation times and selection coefficients given the observations
double FindLikelihood_new (int print_freq, int n_mut, int n_mpops, int last_gen, int find_mean, vector<d_vec> real_counts, vector<int> real_times, vector<double> q0, vector<mutant>& mutations, vector<double>& fact_store) {
	int t=0;
	int n_points=1;
	unsigned int s;
	double logL=0;
	double total_q=0;
	double Lt;
	vector<double> h_pop;
	vector<double> h_freq;
	vector<double> m_freq;
	
	FindZa (n_mut,mutations);

	ofstream mf;
	if (print_freq==1){
		mf.open("Model_frequencies.out");
	}
	for (s=0;s<real_times.size()-1;s++){ 
		t=real_times[s];
		if (t>last_gen) {break;}
		Lt=0;
		
		//Find real frequencies of marker populations at time t - sum of the haplotype frequencies
		h_pop.clear();
		FindHaploPops (total_q,t,n_mpops,n_mut,q0,mutations,h_pop);

		//h_pop contains the haplotype populations - now find frequencies
		h_freq.clear();
		FindHaploFreqs (n_mpops,n_mut,total_q,h_pop,h_freq);
	
		//h_freq contains the haplotype frequencies - now find marker frequencies
		m_freq.clear();
		FindMarkerFreqs (t,print_freq,n_mpops,n_mut,mf,h_pop,h_freq,m_freq,mutations);

		//Calculate likelihood
		LikelihoodCalc (s, n_points,logL,real_counts,fact_store,m_freq);
			
	}
	logL=-logL;
	if (find_mean==1) {
		logL=logL/n_points;
	}
	return logL;
}

//Find the log likelihood in the 0 mutant case
double FindLikelihood_nomut (int print_freq, int n_mpops, int last_gen, vector<d_vec> real_counts, vector<int> real_times, vector<double> q0, vector<double>& fact_store) {
	double logL=0;
	int t=0;
	double total_q=0;
	ofstream mf;
	if (print_freq==1){
		mf.open("Model_frequencies.out");
	}
	int n_points=1;
	for (unsigned int s=0;s<real_times.size()-1;s++){ 
		t=real_times[s];
		if (t>last_gen) {break;}
		
		vector<double> h_pop;
		FindHaploPopsNoMut (total_q,t,n_mpops,q0,h_pop);

		//h_pop contains the haplotype populations - now find frequencies
		vector<double> h_freq;
		FindHaploFreqs (n_mpops,0,total_q,h_pop,h_freq);

		//h_freq contains the haplotype frequencies - now find marker frequencies
		vector<double> m_freq;
		
		FindMarkerFreqsNoMut (t,print_freq,n_mpops,mf,h_freq,m_freq);
		
		//Deal with observed frequencies at this time - likelihood
		LikelihoodCalc (s,n_points,logL,real_counts,fact_store,m_freq);

	}
	logL=-logL;

	return logL;
}


//Outputs the optimal mutants found by the code.  A machine readable version is contained in Transfer_data.out
void DoFinalOutput (int alt, double new_mut_size, double best_fit, double fm_size, int n_mut, int last_gen, int n_mpops, int find_mean, vector<mutant>& mutations, vector<d_vec>& real_counts, vector<int>& real_times, vector<double>& start_freq, vector<double> fact_store, int standing) {
	vector<double> start_freqs;
	
	cout << "Program converged :\n";
	cout << "Best likelihood = "  << best_fit << "\n";
	for (int m=0;m<n_mut;m++){
		cout << "Mutation " << m << " " << mutations[m].selection << " " << mutations[m].time << " " << mutations[m].mpop << "\n";
	}
	for (int c=0;c<n_mpops;c++){
		cout << "Start frequency " << start_freq[c] << "\n";
	}
	
	//Now obtain selection values
	for (int c=0;c<n_mpops;c++){
		start_freqs.push_back(start_freq[c]);
	}
	//Set the initial frequencies q0
	vector<double> q0;
	for (int c=0;c<n_mpops;c++){
		q0.push_back(start_freqs[c]);
	}
	for (int c=n_mpops;c<n_mpops+n_mut;c++){
		q0.push_back(new_mut_size);
	}
	
	//Account for variable first mutant size
	if (standing==1) {
		double min_t=10000;
		int ind=-1;
		for (int i=0;i<n_mut;i++){
			if (mutations[i].time<min_t) {
				min_t=mutations[i].time;
				ind=i;
			}
		}
		if (min_t<1) {
			q0[n_mpops+ind]=fm_size;
		}
	}
	
	find_logfact(fact_store,1000);
	int print_freq=1;
	double logL = FindLikelihood_new (print_freq,n_mut,n_mpops,last_gen,find_mean,real_counts,real_times,q0,mutations,fact_store);
	cout << logL << "\n";
	
	WriteTransferData (alt,n_mut,n_mpops,best_fit,fm_size,start_freq,mutations,standing);
	
}	

//Output optimal parameters in machine-readable form
void WriteTransferData (int alt, int n_mut, int n_mpops, double best_fit, double fm_size, vector<double>& start_freq, vector<mutant>& mutations, int standing) {
	ofstream transfer;
	if (alt==1) {
		transfer.open("Transfer_data2.out");
	} else {
		transfer.open("Transfer_data.out");
	}
	transfer << n_mut << "\n";
	for (int m=0;m<n_mut;m++){
		transfer << mutations[m].selection << " " << mutations[m].time << " " << mutations[m].mpop << "\n";
	}
	transfer << n_mpops << "\n";
	for (int c=0;c<n_mpops;c++){
		transfer << start_freq[c] << "\n";
	}
	if (standing==1) {
		transfer << fm_size << "\n";
	}
	transfer << best_fit << "\n";
}

//Output optimal parameters in machine-readable form : here for a number of optimisations
void WriteTransferDataFile (int reps, int n_mut, int n_mpops, vector<mut_data>& mutation_store, int standing) {
	ofstream transfer;
	transfer.open("Transfer_data_multi.out");
	for (int i=0;i<reps;i++){
		transfer << n_mut << "\n";
		for (int m=0;m<n_mut;m++){
			transfer << mutation_store[i].data[m].selection << " " << mutation_store[i].data[m].time << " " << mutation_store[i].data[m].mpop << "\n";
		}
		transfer << n_mpops << "\n";
		for (int c=0;c<n_mpops;c++){
			transfer << mutation_store[i].sf[c] << "\n";
		}
		if (standing==1) {
			transfer << mutation_store[i].fm_size << "\n";
		}
		transfer << mutation_store[i].logL << "\n";
	}
}	



//Reads in the machine readable data from Transfer_data.out.  Used at the start of the sytematic search
void GetTransferData (int& n_mut, vector<mutant>& mutations, int& n_mpops, vector<double>& start_freq, double& best_fit) {
	ifstream transfer;
	transfer.open("Transfer_data.out");
	transfer >> n_mut;
	for (int m=0;m<n_mut;m++){
		mutant mut;
		transfer >> mut.selection;
		transfer >> mut.time;
		transfer >> mut.mpop;
		//Convert mutation times to integers
		if ((mut.time-floor(mut.time))<=0.5) {
			mut.time=floor(mut.time);
		} else {
			mut.time=floor(mut.time)+1;
		}
		mutations.push_back(mut);
	}
	transfer >> n_mpops;
	for (int c=0;c<n_mpops;c++){
		double x;
		transfer >> x;
		start_freq.push_back(x);
	}
	transfer >> best_fit;
}

//Outputs the step sizes for the systematic search
void InitialSystematicOutput (int n_mut, vector<int>change, vector<int>& index) {
	cout << "Change vector ";
	for (unsigned int i=0;i<change.size();i++){
		cout << change[i] << " ";
	}
	cout << "\n";
	
	for (int i=0;i<n_mut;i++){
		index.push_back(0);
	}
	
	index[0]=-1;
}	


